lists.openwall.net   lists  /  announce  owl-users  owl-dev  john-users  john-dev  passwdqc-users  yescrypt  popa3d-users  /  oss-security  kernel-hardening  musl  sabotage  tlsify  passwords  /  crypt-dev  xvendor  /  Bugtraq  Full-Disclosure  linux-kernel  linux-netdev  linux-ext4  linux-hardening  linux-cve-announce  PHC 
Open Source and information security mailing list archives
 
Hash Suite: Windows password security audit tool. GUI, reports in PDF.
[<prev] [next>] [day] [month] [year] [list]
Date:	Thu, 24 Mar 2011 13:48:02 -0400
From:	Joe Korty <joe.korty@...r.com>
To:	paulmck@...ux.vnet.ibm.com
Cc:	fweisbec@...il.com, peterz@...radead.org, laijs@...fujitsu.com,
	mathieu.desnoyers@...icios.com, dhowells@...hat.com,
	loic.minier@...aro.org, dhaval.giani@...il.com, tglx@...utronix.de,
	josh@...htriplett.org, houston.jim@...cast.net,
	andi@...stfloor.org, linux-kernel@...r.kernel.org
Subject: [PATCH 11/24] jrcu: use xchg to toggle rcu_which

Use xchg rather than wmb()s to change the value
of rcu_which.

This may be better than wmb()s as xchg will not
let the insn counter advance past the xchg until
the exchanged result is visible to other cpus.

Signed-off-by: Joe Korty <joe.korty@...r.com>

Index: b/kernel/jrcu.c
===================================================================
--- a/kernel/jrcu.c
+++ b/kernel/jrcu.c
@@ -371,14 +371,16 @@ static void __rcu_delimit_batches(struct
 		else
 			rd->wait = 0;
 	}
+	smp_wmb(); /* just paranoia, the below xchg should do this on all archs */
 
 	/*
 	 * Swap current and previous lists.  The other cpus must not see this
 	 * out-of-order w.r.t. the above emptying of each cpu's previous list,
 	 * hence the smp_wmb.
 	 */
-	smp_wmb();
-	rcu_which = prev;
+	xchg(&rcu_which, prev); /* only place where rcu_which is written to */
 	rcu_stats.nbatches++;
 }
 
@@ -371,14 +371,16 @@ static void __rcu_delimit_batches(struct
 
 	/*
-	 * Swap current and previous lists.  The other cpus must not see this
-	 * out-of-order w.r.t. the above emptying of each cpu's previous list,
-	 * hence the smp_wmb.
+	 * Swap current and previous lists.  The other cpus must not
+	 * see this out-of-order w.r.t. the above emptying of each cpu's
+	 * previous list.  The xchg accomplishes that and, as a side (but
+	 * seemingly unneeded) bonus, keeps this cpu from advancing its insn
+	 * counter until the results of that xchg are visible on other cpus.
 	 */
--
To unsubscribe from this list: send the line "unsubscribe linux-kernel" in
the body of a message to majordomo@...r.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html
Please read the FAQ at  http://www.tux.org/lkml/

Powered by blists - more mailing lists

Powered by Openwall GNU/*/Linux Powered by OpenVZ